
#ifndef KDEVPLATFORMSHELL_EXPORT_H
#define KDEVPLATFORMSHELL_EXPORT_H

#ifdef KDEVPLATFORMSHELL_STATIC_DEFINE
#  define KDEVPLATFORMSHELL_EXPORT
#  define KDEVPLATFORMSHELL_NO_EXPORT
#else
#  ifndef KDEVPLATFORMSHELL_EXPORT
#    ifdef KDevPlatformShell_EXPORTS
        /* We are building this library */
#      define KDEVPLATFORMSHELL_EXPORT __attribute__((visibility("default")))
#    else
        /* We are using this library */
#      define KDEVPLATFORMSHELL_EXPORT __attribute__((visibility("default")))
#    endif
#  endif

#  ifndef KDEVPLATFORMSHELL_NO_EXPORT
#    define KDEVPLATFORMSHELL_NO_EXPORT __attribute__((visibility("hidden")))
#  endif
#endif

#ifndef KDEVPLATFORMSHELL_DEPRECATED
#  define KDEVPLATFORMSHELL_DEPRECATED __attribute__ ((__deprecated__))
#endif

#ifndef KDEVPLATFORMSHELL_DEPRECATED_EXPORT
#  define KDEVPLATFORMSHELL_DEPRECATED_EXPORT KDEVPLATFORMSHELL_EXPORT KDEVPLATFORMSHELL_DEPRECATED
#endif

#ifndef KDEVPLATFORMSHELL_DEPRECATED_NO_EXPORT
#  define KDEVPLATFORMSHELL_DEPRECATED_NO_EXPORT KDEVPLATFORMSHELL_NO_EXPORT KDEVPLATFORMSHELL_DEPRECATED
#endif

/* NOLINTNEXTLINE(readability-avoid-unconditional-preprocessor-if) */
#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef KDEVPLATFORMSHELL_NO_DEPRECATED
#    define KDEVPLATFORMSHELL_NO_DEPRECATED
#  endif
#endif

#endif /* KDEVPLATFORMSHELL_EXPORT_H */
