
#ifndef KDEVPLATFORMPROJECT_EXPORT_H
#define KDEVPLATFORMPROJECT_EXPORT_H

#ifdef KDEVPLATFORMPROJECT_STATIC_DEFINE
#  define KDEVPLATFORMPROJECT_EXPORT
#  define KDEVPLATFORMPROJECT_NO_EXPORT
#else
#  ifndef KDEVPLATFORMPROJECT_EXPORT
#    ifdef KDevPlatformProject_EXPORTS
        /* We are building this library */
#      define KDEVPLATFORMPROJECT_EXPORT __attribute__((visibility("default")))
#    else
        /* We are using this library */
#      define KDEVPLATFORMPROJECT_EXPORT __attribute__((visibility("default")))
#    endif
#  endif

#  ifndef KDEVPLATFORMPROJECT_NO_EXPORT
#    define KDEVPLATFORMPROJECT_NO_EXPORT __attribute__((visibility("hidden")))
#  endif
#endif

#ifndef KDEVPLATFORMPROJECT_DEPRECATED
#  define KDEVPLATFORMPROJECT_DEPRECATED __attribute__ ((__deprecated__))
#endif

#ifndef KDEVPLATFORMPROJECT_DEPRECATED_EXPORT
#  define KDEVPLATFORMPROJECT_DEPRECATED_EXPORT KDEVPLATFORMPROJECT_EXPORT KDEVPLATFORMPROJECT_DEPRECATED
#endif

#ifndef KDEVPLATFORMPROJECT_DEPRECATED_NO_EXPORT
#  define KDEVPLATFORMPROJECT_DEPRECATED_NO_EXPORT KDEVPLATFORMPROJECT_NO_EXPORT KDEVPLATFORMPROJECT_DEPRECATED
#endif

/* NOLINTNEXTLINE(readability-avoid-unconditional-preprocessor-if) */
#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef KDEVPLATFORMPROJECT_NO_DEPRECATED
#    define KDEVPLATFORMPROJECT_NO_DEPRECATED
#  endif
#endif

#endif /* KDEVPLATFORMPROJECT_EXPORT_H */
