// This file was generated by ecm_setup_version(): DO NOT EDIT!

#ifndef KDEVELOP_VERSION_H
#define KDEVELOP_VERSION_H

#define KDEVELOP_VERSION_STRING "6.5.260400"
#define KDEVELOP_VERSION_MAJOR 6
#define KDEVELOP_VERSION_MINOR 5
#define KDEVELOP_VERSION_PATCH 260400
#define KDEVELOP_VERSION ((6<<16)|(5<<8)|(260400))

#endif
